@@ -653,6 +653,7 @@ static gchar *default_render_select_target (GdaSqlSelectTarget *target, GdaSqlRe
 static gchar *default_render_select_join (GdaSqlSelectJoin *join, GdaSqlRenderingContext *context, GError **error);
 static gchar *default_render_select_from (GdaSqlSelectFrom *from, GdaSqlRenderingContext *context, GError **error);
 static gchar *default_render_select_order (GdaSqlSelectOrder *order, GdaSqlRenderingContext *context, GError **error);
+static gchar *default_render_distinct (GdaSqlStatementSelect *select, GdaSqlRenderingContext *context, GError **error);
 
 /**
  * gda_statement_to_sql_real:
@@ -714,6 +715,8 @@ gda_statement_to_sql_real (GdaStatement *stmt, GdaSqlRenderingContext *context,
 		context->render_select_from = (GdaSqlRenderingFunc) default_render_select_from;
 	if (!context->render_select_order)
 		context->render_select_order = (GdaSqlRenderingFunc) default_render_select_order;
+	if (!context->render_distinct)
+		context->render_distinct = (GdaSqlRenderingFunc) default_render_distinct;
 
 	cinfo = gda_sql_statement_get_contents_infos (stmt->priv->internal_struct->stmt_type);
 	if (cinfo->check_structure_func && !cinfo->check_structure_func (GDA_SQL_ANY_PART (stmt->priv->internal_struct->contents), 
@@ -1144,6 +1147,36 @@ default_render_compound (GdaSqlStatementCompound *stmt, GdaSqlRenderingContext *
 	return NULL;	
 }
 
+static gchar *
+default_render_distinct (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *context, GError **error)
+{
+	gboolean pretty = context->flags & GDA_STATEMENT_SQL_PRETTY;
+	g_return_val_if_fail (stmt, NULL);
+	g_return_val_if_fail (GDA_SQL_ANY_PART (stmt)->type == GDA_SQL_ANY_STMT_SELECT, NULL);
+
+	if (stmt->distinct) {
+		GString *string;
+		string = g_string_new ("DISTINCT");
+		if (stmt->distinct_expr) {
+			gchar *str;
+			str = context->render_expr (stmt->distinct_expr, context, NULL, NULL, error);
+			if (!str) {
+				g_string_free (string, TRUE);
+				return NULL;
+			}
+			g_string_append (string, " ON (");
+			g_string_append (string, str);
+			g_string_append (string, ") ");
+			g_free (str);
+		}
+		if (pretty)
+			g_string_append_c (string, '\n');
+		return g_string_free (string, FALSE);
+	}
+	else
+		return NULL;
+}
+
 static gchar *
 default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *context, GError **error)
 {
@@ -1158,17 +1191,11 @@ default_render_select (GdaSqlStatementSelect *stmt, GdaSqlRenderingContext *cont
 	string = g_string_new ("SELECT ");
 	/* distinct */
 	if (stmt->distinct) {
-		g_string_append (string, "DISTINCT ");
-		if (stmt->distinct_expr) {
-			str = context->render_expr (stmt->distinct_expr, context, NULL, NULL, error);
-			if (!str) goto err;
-			g_string_append (string, "ON ");
-			g_string_append (string, str);
-			g_string_append_c (string, ' ');
-			g_free (str);
-		}
-		if (pretty)
-			g_string_append_c (string, '\n');
+		str = context->render_distinct (GDA_SQL_ANY_PART (stmt), context, error);
+		if (!str) goto err;
+		g_string_append (string, str);
+		g_string_append_c (string, ' ');
+		g_free (str);
 	}
 	
 	/* selected expressions */
@@ -1476,44 +1503,74 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		if (!str) goto err;
 	}
 	else if (expr->value) {
-		if (expr->value_is_ident && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) &&
-		    g_value_get_string (expr->value)) {
-			gchar **ids_array;
-			gint i;
-			GString *string = NULL;
-			GdaConnectionOptions cncoptions = 0;
-			if (context->cnc)
-				g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
-
-			ids_array = gda_sql_identifier_split (g_value_get_string (expr->value));
-			if (!ids_array) 
-				str = g_value_dup_string (expr->value);
-			else if (!(ids_array[0])) goto err;
-			else {
-				for (i = 0; ids_array[i]; i++) {
-					gchar *tmp;
-					if (!string)
-						string = g_string_new ("");
-					else
-						g_string_append_c (string, '.');
-					tmp = gda_sql_identifier_quote (ids_array[i], context->cnc,
-									context->provider, FALSE,
-					   cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
-					g_string_append (string, tmp);
-					g_free (tmp);
+		if (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) {
+			/* specific treatment for strings, see documentation about GdaSqlExpr's value attribute */
+			const gchar *vstr;
+			vstr = g_value_get_string (expr->value);
+			if (vstr) {
+				if (expr->value_is_ident) {
+					gchar **ids_array;
+					gint i;
+					GString *string = NULL;
+					GdaConnectionOptions cncoptions = 0;
+					if (context->cnc)
+						g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
+					ids_array = gda_sql_identifier_split (vstr);
+					if (!ids_array)
+						str = g_strdup (vstr);
+					else if (!(ids_array[0])) goto err;
+					else {
+						for (i = 0; ids_array[i]; i++) {
+							gchar *tmp;
+							if (!string)
+								string = g_string_new ("");
+							else
+								g_string_append_c (string, '.');
+							tmp = gda_sql_identifier_quote (ids_array[i], context->cnc,
+											context->provider, FALSE,
+											cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
+							g_string_append (string, tmp);
+							g_free (tmp);
+						}
+						g_strfreev (ids_array);
+						str = g_string_free (string, FALSE);
+					}
 				}
-				g_strfreev (ids_array);
-				str = g_string_free (string, FALSE);
+				else {
+					/* we don't have an identifier */
+					if (!g_ascii_strcasecmp (vstr, "default")) {
+						if (is_default)
+							*is_default = TRUE;
+						str = g_strdup ("DEFAULT");
+					}
+					else
+						str = g_strdup (vstr);
+				}
+			}
+			else {
+				str = g_strdup ("NULL");
+				if (is_null)
+					*is_null = TRUE;
 			}
 		}
-		else {
-			str = gda_value_stringify (expr->value);
+		if (!str) {
+			/* use a GdaDataHandler to render the value as valid SQL */
+			GdaDataHandler *dh;
+			if (context->cnc) {
+				GdaServerProvider *prov;
+				prov = gda_connection_get_provider (context->cnc);
+				dh = gda_server_provider_get_data_handler_g_type (prov, context->cnc,
+										  G_VALUE_TYPE (expr->value));
+				if (!dh) goto err;
+			}
+			else
+				dh = gda_data_handler_get_default (G_VALUE_TYPE (expr->value));
+
+			if (dh)
+				str = gda_data_handler_get_sql_from_value (dh, expr->value);
+			else
+				str = gda_value_stringify (expr->value);
 			if (!str) goto err;
-			if (is_null && gda_value_is_null (expr->value))
-				*is_null = TRUE;
-			else if (is_default && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) && 
-				 !g_ascii_strcasecmp (g_value_get_string (expr->value), "default"))
-				*is_default = TRUE;
 		}
 	}
 	else if (expr->func) {
@@ -1580,10 +1637,7 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		str = g_strdup ("NULL");
 	}
 
-	if (!str) {
-		/* TO REMOVE */
-		str = g_strdup ("[...]");
-	}
+	if (!str) goto err;
 
 	if (expr->cast_as) 
 		g_string_append_printf (string, "CAST (%s AS %s)", str, expr->cast_as);
