@@ -86,7 +86,6 @@ _remove_quotes (gchar *str)
 	if ((delim != '\'') && (delim != '"'))
 		return str;
 
-
         total = strlen (str);
         if (str[total-1] == delim) {
 		/* string is correctly terminated by a double quote */
@@ -113,7 +112,17 @@ _remove_quotes (gchar *str)
                                 return str;
                         }
                 }
-                if (*ptr == '\\') {
+                else if (*ptr == '"') {
+                        if (*(ptr+1) == '"') {
+                                g_memmove (ptr+1, ptr+2, total - offset);
+                                offset += 2;
+                        }
+                        else {
+				*str = 0;
+				return str;
+                        }
+                }
+		else if (*ptr == '\\') {
                         if (*(ptr+1) == '\\') {
                                 g_memmove (ptr+1, ptr+2, total - offset);
                                 offset += 2;
@@ -165,7 +174,7 @@ gda_sql_identifier_add_quotes (const gchar *str)
 	*retval = '"';
 	for (rptr = retval+1, sptr = str; *sptr; sptr++, rptr++) {
 		if (*sptr == '"') {
-			*rptr = '\\';
+			*rptr = '"';
 			rptr++;
 			*rptr = *sptr;
 		}
